
import java.util.*;
/**
 * Verfolgt das Monster bis zu einem Abstand von 10 und bewegt sich dann zufällig.
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
class Chaot extends Hindernis
{
    /** x-Startposition merken */
    int xStart;
    /** y-Startposition merken */
    int yStart;
    /** Der Zufallsgenerator */
    Random zufall;
    
    /**
     * Der Konstruktor legt Position, Größe und Farbe fest
     * @param x x-Koordinate des Rechtecks
     * @param y y-Koordinate des Rechtecks
     * @param rahmen das Rahmenobjekt des Spiels
     */
    Chaot (int x, int y, Rahmen rahmen)
    {
        super(x, y, rahmen);
        FigurErzeugen("blau");
        GrößeSetzen(rahmen.basisGröße);
        xStart = x;
        yStart = y;
        zufall = new Random();
    }
    
    /**
     * Erzeugt das Figurenbild in der gewünschten Farbe
     * @param farbe: die gewünschte Farbe
     */
    private void FigurErzeugen(String farbe)
    {
        EigeneFigurLöschen();
        FigurteilFestlegenEllipse(-15, -50, 30, 30, farbe);
        FigurteilFestlegenDreieck(-25, 45, 0, -50, 25, 45, farbe);
        FigurteilFestlegenRechteck(-15, -35, 30, 80, farbe);
        FigurteilFestlegenEllipse(-25, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(-15, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(-5, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(5, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(15, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(-11, -35, 9, 9, "schwarz");
        FigurteilFestlegenEllipse(2, -35, 9, 9, "schwarz");
    }
    
    /**
     * Der Chaot nimmt dem Monster ein Leben ab.
     * @param monster das Monster
     */
    @Override void Reagieren(Monster monster)
    {
        if (Berührt(monster))
        {
            monster.LebenNehmen();
            AufStartpositionSetzen();
        }
    }
    
    /**
     * Setzt den Chaot auf die Startposition zurück
     */
    private void AufStartpositionSetzen()
    {
        PositionSetzen(xStart * rahmen.basisGröße +  rahmen.basisGröße / 2, yStart * rahmen.basisGröße +  rahmen.basisGröße / 2);
    }
    
    /**
     * Berechnet den Kästchenabstand der gegebenen Koordinaten zum Monster
     * @param x x-Koordinate des Punktes, zu dem der Abstand berechnet wird
     * @param y y-Koordinate des Punktes, zu dem der Abstand berechnet wird
     * @return Kästchenabstand
     */
    private int AbstandGeben(int x, int y)
    {
        return (Math.abs(x - rahmen.MonsterGeben().XPositionGeben()) + Math.abs(y - rahmen.MonsterGeben().YPositionGeben())) / rahmen.basisGröße;
    }
    
    /**
     * Der Chaot bewegt sich direkt auf das Monster zu,
     * bei nahem Abstand zufällig
     */
    private void Bewegen()
    {
        int minAbstand = 100000;
        int xOpt = XPositionGeben();
        int yOpt = YPositionGeben();
        if (AbstandGeben(XPositionGeben(), YPositionGeben()) <= 10)
        {
            do
            {
                switch (zufall.nextInt(4))
                {
                    case 0:
                        xOpt = XPositionGeben() + rahmen.basisGröße;
                        yOpt = YPositionGeben();
                        break;
                    case 1:
                        xOpt = XPositionGeben() - rahmen.basisGröße;
                        yOpt = YPositionGeben();
                        break;
                    case 2:
                        xOpt = XPositionGeben();
                        yOpt = YPositionGeben() + rahmen.basisGröße;
                        break;
                    case 3:
                        xOpt = XPositionGeben();
                        yOpt = YPositionGeben() - rahmen.basisGröße;
                        break;
                }
            } while (rahmen.IstMauer(xOpt, yOpt));
        }
        else
        {
            if (!rahmen.IstMauer(XPositionGeben(), YPositionGeben() - rahmen.basisGröße))
            {
                xOpt = XPositionGeben();
                yOpt = YPositionGeben() - rahmen.basisGröße;
                minAbstand = AbstandGeben(xOpt, yOpt);
            }
            if ((AbstandGeben(XPositionGeben(), YPositionGeben() + rahmen.basisGröße) < minAbstand) && (!rahmen.IstMauer(XPositionGeben(), YPositionGeben() + rahmen.basisGröße)))
            {
                xOpt = XPositionGeben();
                yOpt = YPositionGeben() + rahmen.basisGröße;
                minAbstand = AbstandGeben(xOpt, yOpt);
            }
            if ((AbstandGeben(XPositionGeben() + rahmen.basisGröße, YPositionGeben()) < minAbstand) && (!rahmen.IstMauer(XPositionGeben() + rahmen.basisGröße, YPositionGeben())))
            {
                xOpt = XPositionGeben() + rahmen.basisGröße;
                yOpt = YPositionGeben();
                minAbstand = AbstandGeben(xOpt, yOpt);
            }
            if ((AbstandGeben(XPositionGeben() - rahmen.basisGröße, YPositionGeben()) < minAbstand) && (!rahmen.IstMauer(XPositionGeben() - rahmen.basisGröße, YPositionGeben())))
            {
                xOpt = XPositionGeben() - rahmen.basisGröße;
                yOpt = YPositionGeben();
                minAbstand = AbstandGeben(xOpt, yOpt);
            }
        }
        PositionSetzen(xOpt, yOpt);
    }
    
    /**
     * Der Chaot geht
     */
    @Override void AktionAusführen()
    {
        Bewegen();
        if (Berührt(rahmen.MonsterGeben()))
        {
            rahmen.MonsterGeben().LebenNehmen();
            AufStartpositionSetzen();
        }
    }
}
