
/**
 * Geht konsequent zu einem Punkt 4 Einheiten vor dem Monster.
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
class Fallensteller extends Hindernis
{
    /** x-Startposition merken */
    int xStart;
    /** y-Startposition merken */
    int yStart;
    
    /**
     * Der Konstruktor legt Position, Größe und Farbe fest
     * @param x x-Koordinate des Rechtecks
     * @param y y-Koordinate des Rechtecks
     * @param rahmen das Rahmenobjekt des Spiels
     */
    Fallensteller(int x, int y, Rahmen rahmen)
    {
        super(x, y, rahmen);
        FigurErzeugen("grün");
        GrößeSetzen(rahmen.basisGröße);
        xStart = x;
        yStart = y;
    }
    
    private void FigurErzeugen(String farbe)
    {
        EigeneFigurLöschen();
        FigurteilFestlegenEllipse(-15, -50, 30, 30, farbe);
        FigurteilFestlegenDreieck(-25, 45, 0, -50, 25, 45, farbe);
        FigurteilFestlegenRechteck(-15, -35, 30, 80, farbe);
        FigurteilFestlegenEllipse(-25, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(-15, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(-5, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(5, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(15, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(-11, -35, 9, 9, "schwarz");
        FigurteilFestlegenEllipse(2, -35, 9, 9, "schwarz");
    }
    
    /**
     * Der Fallensteller nimmt dem Monster ein Leben ab.
     * @param monster das Monster
     */
    @Override void Reagieren(Monster monster)
    {
        if (Berührt(monster))
        {
            monster.LebenNehmen();
            AufStartpositionSetzen();
        }
    }
    
    /**
     * Setzt den Fallensteller auf die Startposition zurück
     */
    private void AufStartpositionSetzen()
    {
        PositionSetzen(xStart * rahmen.basisGröße +  rahmen.basisGröße / 2, yStart * rahmen.basisGröße + rahmen.basisGröße / 2);
    }
    
    /**
     * Berechnet den Kästchenabstand der gegebenen Koordinatenpaare
     * @param x x-Koordinate des 1. Punktes
     * @param y y-Koordinate des 1. Punktes
     * @param x2 x-Koordinate des 2. Punktes
     * @param y2 y-Koordinate des 2. Punktes
     * @return Kästchenabstand
     */
    private int AbstandGeben (int x, int y, int x2, int y2)
    {
        return (Math.abs(x - x2) + Math.abs(y - y2)) / rahmen.basisGröße;
    }
    
    /**
     * Der Fallensteller bewegt sich auf einen Punkt vor dem Monster zu
     */
    private void Bewegen()
    {
        int minAbstand = 100000;
        int xZiel = rahmen.MonsterGeben().XPositionGeben();
        int yZiel = rahmen.MonsterGeben().YPositionGeben();
        switch (rahmen.MonsterGeben().WinkelGeben())
        {
            case 0:
                xZiel = xZiel + 4 * rahmen.basisGröße;
                break;
            case 90:
                yZiel = yZiel - 4 * rahmen.basisGröße;
                break;
            case 180:
                xZiel = xZiel - 4 * rahmen.basisGröße;
                break;
            case 270:
                yZiel = yZiel + 4 * rahmen.basisGröße;
                break;
        }
        int xOpt = XPositionGeben();
        int yOpt = YPositionGeben();
        if (!rahmen.IstMauer(XPositionGeben(), YPositionGeben() - rahmen.basisGröße))
        {
            xOpt = XPositionGeben();
            yOpt = YPositionGeben() - rahmen.basisGröße;
            minAbstand = AbstandGeben(xOpt, yOpt, xZiel, yZiel);
        }
        if ((AbstandGeben(XPositionGeben(), YPositionGeben() + rahmen.basisGröße, xZiel, yZiel) < minAbstand) && (!rahmen.IstMauer(XPositionGeben(), YPositionGeben() + rahmen.basisGröße)))
        {
            xOpt = XPositionGeben();
            yOpt = YPositionGeben() + rahmen.basisGröße;
            minAbstand = AbstandGeben(xOpt, yOpt, xZiel, yZiel);
        }
        if ((AbstandGeben(XPositionGeben() + rahmen.basisGröße, YPositionGeben(), xZiel, yZiel) < minAbstand) && (!rahmen.IstMauer(XPositionGeben() + rahmen.basisGröße, YPositionGeben())))
        {
            xOpt = XPositionGeben() + rahmen.basisGröße;
            yOpt = YPositionGeben();
            minAbstand = AbstandGeben(xOpt, yOpt, xZiel, yZiel);
        }
        if ((AbstandGeben(XPositionGeben() - rahmen.basisGröße, YPositionGeben(), xZiel, yZiel) < minAbstand) && (!rahmen.IstMauer(XPositionGeben() - rahmen.basisGröße, YPositionGeben())))
        {
            xOpt = XPositionGeben() - rahmen.basisGröße;
            yOpt = YPositionGeben();
            minAbstand = AbstandGeben(xOpt, yOpt, xZiel, yZiel);
        }
        PositionSetzen(xOpt, yOpt);
    }
    
    /**
     * Der Fallensteller geht
     */
    @Override void AktionAusführen()
    {
        Bewegen();
        if (Berührt(rahmen.MonsterGeben()))
        {
            rahmen.MonsterGeben().LebenNehmen();
            AufStartpositionSetzen();
        }
    }
}
