
/**
 * Verfolgt das Monster konsequent.
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
class Jaeger extends Hindernis
{
    /** x-Startposition merken */
    int xStart;
    /** y-Startposition merken */
    int yStart;
    
    /**
     * Der Konstruktor legt Position, Größe und Farbe fest
     * @param x x-Koordinate des Rechtecks
     * @param y y-Koordinate des Rechtecks
     * @param rahmen das Rahmenobjekt des Spiels
     */
    Jaeger(int x, int y, Rahmen rahmen)
    {
        super(x, y, rahmen);
        FigurErzeugen("rot");
        GrößeSetzen(rahmen.basisGröße);
        xStart = x;
        yStart = y;
    }
    
    void FigurErzeugen(String farbe)
    {
        EigeneFigurLöschen();
        FigurteilFestlegenEllipse(-15, -50, 30, 30, farbe);
        FigurteilFestlegenDreieck(-25, 45, 0, -50, 25, 45, farbe);
        FigurteilFestlegenRechteck(-15, -35, 30, 80, farbe);
        FigurteilFestlegenEllipse(-25, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(-15, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(-5, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(5, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(15, 40, 10, 10, farbe);
        FigurteilFestlegenEllipse(-11, -35, 9, 9, "schwarz");
        FigurteilFestlegenEllipse(2, -35, 9, 9, "schwarz");
    }
    
    /**
     * Der Jäger nimmt dem Monster ein Leben ab.
     * @param monster das Monster
     */
    @Override void Reagieren(Monster monster)
    {
        if (Berührt(monster))
        {
            monster.LebenNehmen();
            AufStartpositionSetzen();
        }
    }
    
    /**
     * Setzt den Jäger auf die Startposition zurück
     */
    private void AufStartpositionSetzen()
    {
        PositionSetzen(xStart * rahmen.basisGröße +  rahmen.basisGröße / 2, yStart * rahmen.basisGröße +  rahmen.basisGröße / 2);
    }
    
    /**
     * Berechnet den Kästchenabstand der gegebenen Koordinaten zum Monster
     * @param x x-Koordinate des Punktes, zu dem der Abstand berechnet wird
     * @param y y-Koordinate des Punktes, zu dem der Abstand berechnet wird
     * @return Kästchenabstand
     */
    private int AbstandGeben(int x, int y)
    {
        return (Math.abs(x - rahmen.MonsterGeben().XPositionGeben()) + Math.abs(y - rahmen.MonsterGeben().YPositionGeben())) / rahmen.basisGröße;
    }
    
    /**
     * Der Jäger bewegt sich direkt auf das Monster zu
     */
    private void Bewegen()
    {
        int minAbstand = 100000;
        int xOpt = XPositionGeben();
        int yOpt = YPositionGeben();
        if (!rahmen.IstMauer(XPositionGeben(), YPositionGeben() - rahmen.basisGröße))
        {
            xOpt = XPositionGeben();
            yOpt = YPositionGeben() - rahmen.basisGröße;
            minAbstand = AbstandGeben(xOpt, yOpt);
        }
        if ((AbstandGeben(XPositionGeben(), YPositionGeben() + rahmen.basisGröße) < minAbstand) && (!rahmen.IstMauer(XPositionGeben(), YPositionGeben() + rahmen.basisGröße)))
        {
            xOpt = XPositionGeben();
            yOpt = YPositionGeben() + rahmen.basisGröße;
            minAbstand = AbstandGeben(xOpt, yOpt);
        }
        if ((AbstandGeben(XPositionGeben() + rahmen.basisGröße, YPositionGeben()) < minAbstand) && (!rahmen.IstMauer(XPositionGeben() + rahmen.basisGröße, YPositionGeben())))
        {
            xOpt = XPositionGeben() + rahmen.basisGröße;
            yOpt = YPositionGeben();
            minAbstand = AbstandGeben(xOpt, yOpt);
        }
        if ((AbstandGeben(XPositionGeben() - rahmen.basisGröße, YPositionGeben()) < minAbstand) && (!rahmen.IstMauer(XPositionGeben() - rahmen.basisGröße, YPositionGeben())))
        {
            xOpt = XPositionGeben() - rahmen.basisGröße;
            yOpt = YPositionGeben();
            minAbstand = AbstandGeben(xOpt, yOpt);
        }
        PositionSetzen(xOpt, yOpt);
    }
    
    /**
     * Der Jäger geht
     */
    @Override void AktionAusführen()
    {
        Bewegen();
        if (Berührt(rahmen.MonsterGeben()))
        {
            rahmen.MonsterGeben().LebenNehmen();
            AufStartpositionSetzen();
        }
    }
}
