
/**
 * Verwaltet das Monster
 * 
 * @author Albert Wiedemann
 * @version 1.0
 */
class Monster extends Figur
{
    /** Anzahl der restlichen Leben */
    private int leben;
    /** Mundstellung */
    private boolean mundIstZu;
    /** Rahmenobjekt */
    private Rahmen rahmen;
    
    /**
     * Legt die Figur und die Mundstellung fest.
     */
    Monster()
    {
        super();
        rahmen = null;
        leben = 0;
        mundIstZu = true;
        Schließen("gelb");
    }
    
    /**
     * Legt die Figur, die Mundstellung, die Größe und die Position fest.
     * @param x x-Position des Monsters
     * @param y y-Position des Monsters
     * @param rahmen das Rahmenobjekt
     * @param leben Anzahl der Leben beim Start
     */
    Monster(int x, int y, Rahmen rahmen, int leben)
    {
        this();
        PositionSetzen(x * rahmen.basisGröße + rahmen.basisGröße / 2, y * rahmen.basisGröße + rahmen.basisGröße / 2);
        this.rahmen = rahmen;
        GrößeSetzen(rahmen.basisGröße);
        this.leben = leben;
    }
    
    /**
     * Stellt das gelb gefärbte Monster mit geschlossenem Mund dar.
     */
    void GeschlossenGelb()
    {
        EigeneFigurLöschen();
        FigurteilFestlegenEllipse(-50, -50, 100, 100, "gelb");
        FigurteilFestlegenRechteck(0, -2, 50, 4, "schwarz");
        FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz");
    }

    /**
     * Stellt das orange gefärbte Monster mit geschlossenem Mund dar.
     */
    void GeschlossenOrange()
    {
        EigeneFigurLöschen();
        FigurteilFestlegenEllipse(-50, -50, 100, 100, "orange");
        FigurteilFestlegenRechteck(0, -2, 50, 4, "schwarz");
        FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz");
    }

    /**
     * Stellt das rot gefärbte Monster mit geschlossenem Mund dar.
     */
    void GeschlossenRot()
    {
        EigeneFigurLöschen();
        FigurteilFestlegenEllipse(-50, -50, 100, 100, "rot");
        FigurteilFestlegenRechteck(0, -2, 50, 4, "schwarz");
        FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz");
    }

    /**
     * Stellt das grün gefärbte Monster mit geschlossenem Mund dar.
     */
    void GeschlossenGrün()
    {
        EigeneFigurLöschen();
        FigurteilFestlegenEllipse(-50, -50, 100, 100, "grün");
        FigurteilFestlegenRechteck(0, -2, 50, 4, "schwarz");
        FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz");
    }

    /**
     * Stellt das Monster in der angegebenen Farbe mit offenem Mund dar.
     * @param farbe die gewünschte Farbe
     */
    void Offen(String farbe)
    {
        EigeneFigurLöschen();
        FigurteilFestlegenEllipse(-50, -50, 100, 100, farbe);
        FigurteilFestlegenDreieck(0, 0, 50, -10, 50, 10, "schwarz");
        FigurteilFestlegenEllipse(10, -40, 20, 20, "schwarz");
    }
    
    /**
     * Stellt das in einer der Farben gelb, orange, rot bzw. grün mit geschlossenem Mund dar.
     * @param farbe die gewünschte Farbe
     */
    void Schließen(String farbe)
    {
        if (farbe == "gelb")
        {
            GeschlossenGelb();
        }
        if (farbe == "orange")
        {
            GeschlossenOrange();
        }
        if (farbe == "rot")
        {
            GeschlossenRot();
        }
        if (farbe == "grün")
        {
            GeschlossenGrün();
        }
    }
    
    /**
     * Stellt das in einer der Farben gelb, orange, rot bzw. grün mit offenem Mund dar.
     * @param farbe die gewünschte Farbe
     */
    void Öffnen(String farbe)
    {
        if ((farbe == "gelb") || (farbe == "orange") || (farbe == "rot") || (farbe == "grün"))
        {
            Offen(farbe);
        }
    }
    
    /**
     * Meldet die Anzahl der Leben
     * @return Anzahl der Leben
     */
    int LebenGeben()
    {
        return leben;
    }
    
    /**
     * Reduziert die Zahl der Leben um 1
     */
    void LebenNehmen()
    {
        leben = leben - 1;
        rahmen.StatusÄndern();
    }
    
    /**
     * Die Aktionsmethode für gedrückte Sondertasten.
     * @param taste KeyCode der gedrückten Taste
     */
    @Override void SonderTasteGedrückt(int taste)
    {
        switch (taste)
        {
            case 37:
                Drehen(90);
                break;
            case 38:
                Gehen(rahmen.basisGröße);
                for (Hindernis h: rahmen.HindernisseGeben())
                {
                    if (Berührt(h))
                    {
                        h.Reagieren(this);
                    }
                }
                break;
            case 39:
                Drehen(-90);
                break;
            case 40:
                Drehen(180);
                break;
        }
    }
    
    /**
     * Das Monster geht
     */
    @Override void AktionAusführen()
    {
        if (mundIstZu)
        {
            Öffnen("gelb");
            mundIstZu = false;
        }
        else
        {
            Schließen("gelb");
            mundIstZu = true;
        }
        if ((XPositionGeben() == rahmen.basisGröße / 2) && (WinkelGeben() == 180))
        {
            Gehen(- 27 * rahmen.basisGröße);
        }
        else if ((XPositionGeben() == 27 * rahmen.basisGröße + rahmen.basisGröße / 2) && (WinkelGeben() == 0))
        {
            Gehen(- 27 * rahmen.basisGröße);
        }
        else
        {
            Gehen(rahmen.basisGröße);
        }
        for (Hindernis h: rahmen.HindernisseGeben())
        {
            if (Berührt(h))
            {
                h.Reagieren(this);
            }
        }
    }
}
