
import java.util.*;
/**
 * Beschreibt einen Raum des Gewölbes.
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
class Raum
{
    /** Die Nummer des Raums */
    int nummer;
    /** die x-Koordinate des Raums in der Landkarte */
    final int x;
    /** die v-Koordinate des Raums in der Landkarte */
    final int y;
    /** Markierung, dass der Raum schon mal betreten wurde */
    boolean betreten;
    /** Markierung, dass in dem Raum schon mal nach dem Ausgang gesucht wurde */
    boolean ausgangGesucht;
    /** Die Türen zu den anderen Räumen */
    Tueren türen;
    /** Die Schätze im Raum */
    ArrayList<Schatz> schätze;
    /** Das Monster im Raum */
    Monster monster;

    /**
     * der Konstruktor legt die Koordinaten und die Nummer fest.
     * @param x x-Koordinate
     * @param y y-Koordinate
     * @param n Raumnummer
     */
    Raum(int n, int x, int y)
    {
        nummer = n;
        this.x = x;
        this.y = y;
        betreten = false;
        ausgangGesucht = false;
        türen = new Tueren();
        schätze = new ArrayList<Schatz>();
        monster = null;
    }

    /**
     * Meldet die (erreichbaren) Raum mit der angegebenen Nummer.
     * @param nummer die gewünschte Raumnummer
     * @return den zugehörigen Raum oder null
     */
    Raum TürNachGeben(int nummer)
    {
        return türen.TürNachGeben(nummer);
    }
    
    /**
     * Trägt eine neue Tür in die Türliste ein
     * @param raum der Raum, zu dem die Tür führen soll
     */
    void NeueTürEintragen(Raum nach)
    {
        türen.NeueTürEintragen(nach);
    }

    /**
     * Meldet, ob der Raum Schätze hat
     * @reuturn wahr, wenn mindestens ein Schatz vorhanden ist
     */
    boolean SchatzVorhanden()
    {
        return !schätze.isEmpty();
    }

    /*%%%%%
    Schatz HoleSchaetze()
    {
        Schatz hilf = schaetze;
        schaetze = null;
        return hilf;
    };
    */

    /**
     * Lagert den Schatz in diesem Raum
     * @param schatz der zu lagernde Schatz
     */
    void SchatzLagern(Schatz schatz)
    {
        schätze.add(schatz);
    }
    
    /**
     * Gibt einen zufälligen Raum aus dem Gewölbe zurück
     * @param zähler Zähler, wie weit (noch) gesucht werden soll.
     * @return Zufallsraum
     */
    Raum ZufallsRaumGeben(int zähler)
    {
        if (zähler == 0)
        {
            return this;
        }
        else
        {
            return türen.TürNummerGeben(Zufall.zufall.GanzzahlGeben(türen.AnzahlTürenGeben()));
        }
    }
    
    /**
     * Versteckt den Schatz irgendwo im Gewölbe
     * @param schatz der zu versteckende Schatz
     */
    void SchatzVerstecken(Schatz akt)
    {
        Raum ziel;
        do
            ziel = ZufallsRaumGeben(50);
        while (ziel == this);
        ziel.SchatzLagern(akt);
    }
    
    /**
     * Tregt das Betreten des Raumes und gegebenenfalls die stattgefundene Ausgangssuche ein
     * @param ausgangOk gibt an, ob auch nach dem Ausgang gesucht wird
     */
    void BetretenEintragen(boolean ausgangOk)
    {
        betreten = true;
        ausgangGesucht |= ausgangOk;
    }
}
