
/**
 * Das Monster Schlange.
 * Die Schlange klaut dem Spieler Stärke, wenn er keine genügend starke Handwaffe hat.
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
class Schlange extends Monster
{

    /**
     * Der Konstruktor besetzt das Spiel
     * @param sp das Spiel, in dem das Objekt verwendet wird.
     */
    Schlange(Spiel sp)
    {
        super(sp);
    }

    /**
     * Meldet den Namen des Objekts zurück
     * @return name
     */
    @Override String NameGeben()
    {
        return "Schlange";
    }
    
    /**
     * Einleitung des Kampfs.
     * @param sp der angegriffene Spieler
     */
    @Override void KampfBeginnen(Spieler sp)
    {
        super.KampfBeginnen(sp);
        if (sp.schutz == null)
        {
            sp.StärkeÄndern(- sp.stärke / 2);
        }
        else if (sp.schutz.AbwehrstärkeGeben() <= 5 + Zufall.zufall.GanzzahlGeben(7))
        {
            sp.StärkeÄndern(-sp.stärke / 4);
        }
        else
        {
            sp.schutz.AnwendungEintragen();
        }
    }
    
    /**
     * Kampf des Spielers mit dem Monster
     * @param sp der angegriffene Spieler
     * @return wahr, wenn der Spieler den Kampf gewonnen hat.
     */
    @Override boolean KampfDurchführen(Spieler sp)
    {
        if ((sp.handWaffe != null) && (sp.stärke + sp.handWaffe.WaffenstärkeGeben() + Zufall.zufall.GanzzahlGeben(5) >= 15))
        {
            sp.handWaffe.AnwendungEintragen();
            spiel.BotschaftSenden("Diese Schlange ist aufgeräumt");
            sp.wo.monster = new LeeresMonster(spiel);
            return true;
        }
        else
        {
            sp.StärkeÄndern(-1);
            if ((sp.schutz == null) || (sp.schutz.AbwehrstärkeGeben() <= 5 + Zufall.zufall.GanzzahlGeben(7)))
            {
                sp.StärkeÄndern(-2);
            }
            return false;
        }
    }

    /**
     * Kampfabbruch durch Flucht des Spielers.
     * Standardmäßig keine Aktion
     * @param sp der angegriffene Spieler
     */
    @Override void KampfAbbrechen(Spieler sp)
    {
        if (!sp.schätze.isEmpty())
        {
            sp.wo.SchatzLagern(sp.schätze.remove(Zufall.zufall.GanzzahlGeben(sp.schätze.size())));
            if (!sp.schätze.isEmpty())
            {
                sp.wo.SchatzLagern(sp.schätze.remove(Zufall.zufall.GanzzahlGeben(sp.schätze.size())));
            }
        }
    }
}
