
import java.io.*;
import java.util.*;
/**
 * Verwaltet das Spielerobjekt
 * 
 * @author Albert Wiedemann
 * @version 1.0
 */
class Spieler extends SpielObjekt
{
    /** das maximal tragbare Geewicht */
    private final double maxGewicht = 90.0;
    /** die maximale Stärke */
    private final int maxStärke = 20;
    /** der Raum, in dem sich der Spieler zur Zeit befindet */
    Raum wo;
    /** die Schätze des Spielers */
    ArrayList<Schatz> schätze;
    /** die Handwaffe */
    AngriffsWaffe handWaffe;
    /** die Schutzwaffe */
    SchutzWaffe schutz;
    /** die aktuelle Stärke */
    int stärke;
    /** die Erfahrung */
    private int erfahrung;

    /**
     * Der Konstruktor setzt die Rahmendaten
     * @param spiel das Spiel, in dem der Spieler agiert
     * @param start der Startraum des Spielers
     */
    Spieler(Spiel spiel, Raum start)
    {
        super(spiel);
        wo = start;
        wo.betreten = true;
        schätze = new ArrayList<Schatz>();
        handWaffe = null;
        schutz = null;
        stärke = maxStärke;
        erfahrung = 0;
    }

    /**
     * Meldet den Namen des Objekts zurück
     * @return name
     */
    @Override String NameGeben()
    {
        return "Spieler";
    }

    /**
     * Geht zum angegebenen Raum
     * @param nummer Nummer des Zielraums
     * @return true, wenn der Raum erreichbar ist
     */
    boolean NachRaumGehen(int nummer)
    {
        Raum neu = wo.TürNachGeben(nummer);
        if (neu != null)
        {
            wo = neu;
            wo.BetretenEintragen(AnzahlSchlüsselGeben() == 3);
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * ermittelt die Anzahl der Schlüssel des Spielers
     * @return Anzahl der Schlüssel
     */
    int AnzahlSchlüsselGeben()
    {
        int anz = 0;
        for (Schatz akt: schätze)
        {
            if (akt instanceof Schluessel)
            {
                anz += 1;
            }
        }
        return anz;
    }
    
    /**
     * Prüft, ob alle Schätze getragen werden können und lässt gegebenenfalls welche da.
     * Sonderschätze werden immer behalten, hand- und Schutzwaffe wenn möglich.
     * Die schwersten Schätze werden zuerst genommen.
     */
    void SchätzeTesten()
    {
        double max, gewicht;
        gewicht = 0.0;
        for (Schatz akt: schätze)
        {
            gewicht += akt.GewichtGeben();
        }
        if (gewicht > maxGewicht)
        {
            ArrayList<Schatz> opt;
            ArrayList<Schatz> rest;
            ArrayList<Schatz> immer;
            immer = new ArrayList<Schatz>();
            rest = new ArrayList<Schatz>();
            opt = new ArrayList<Schatz>();
            max = maxGewicht;
            int pos = 0;
            while (pos < schätze.size())
            {
                Schatz akt = schätze.get(pos);
                if ((akt.GewichtGeben() <= 0.0) || (akt instanceof Schluessel))
                {
                    immer.add(akt);
                    schätze.remove(pos);
                    max -= akt.GewichtGeben();
                }
                else
                {
                    pos = pos + 1;
                }
            }
            if ((handWaffe != null) && (handWaffe.GewichtGeben() <= max))
            {
                schätze.remove(handWaffe);
                max -= handWaffe.GewichtGeben();
                immer.add(handWaffe);
            }
            if ((schutz != null) && (schutz.GewichtGeben() <= max))
            {
                schätze.remove(schutz);
                max -= schutz.GewichtGeben();
                immer.add(schutz);
            }
            while (schätze.size() > 0)
            {
                double aktGewicht = 0.0;
                Schatz top = null;
                for (Schatz akt: schätze)
                {
                    if (akt.GewichtGeben() > aktGewicht)
                    {
                        top = akt;
                        aktGewicht = akt.GewichtGeben();
                    }
                }
                if (top.GewichtGeben() <= max)
                {
                    opt.add(top);
                    max -= top.GewichtGeben();
                }
                else
                {
                    rest.add(top);
                }
                schätze.remove(top);
            }
            schätze = opt;
            for (Schatz akt: immer)
            {
                schätze.add(akt);
            }
            for (Schatz akt: rest)
            {
                wo.SchatzLagern(akt);
            }
        }
        if (schätze.isEmpty())
        {
            handWaffe = null;
            schutz = null;
        }
        else
        {
            if (!schätze.contains(handWaffe))
            {
                handWaffe = null;
            }
            if (!schätze.contains(schutz))
            {
                schutz = null;
            }
        }
    }

    /**
     * Lässt den angegebenen Schatz da
     * @param nummer laufende Nummer des Schatzes
     */
    void SchatzDalassen(int nummer)
    {
        if (nummer < schätze.size())
        {
            Schatz hilf = schätze.remove(nummer);
            wo.SchatzLagern(hilf);
            SchätzeTesten();
            InventarListeEintragen();
            RaumSchätzeEintragen();
        }
    }
    
    /**
     * Nimmt den angegebenen Schatz mit
     * @param nummer laufende Nummer des Schatzes
     */
    void SchatzMitnehmen(int nummer)
    {
        
        if (nummer < wo.schätze.size())
        {
            double gewicht = 0.0;
            double mentalGewicht = 0.0;
            Schatz hilf;
            for (Schatz akt: schätze)
            {
                gewicht += akt.GewichtGeben();
                if (akt instanceof Spruch)
                {
                    mentalGewicht += ((Spruch) akt).MentalgewichtGeben();
                }
            }
            hilf = wo.schätze.get(nummer);
            if ((gewicht + hilf.GewichtGeben() <= maxGewicht) && (hilf instanceof Spruch ? mentalGewicht + ((Spruch) hilf).MentalgewichtGeben() <= 20 + erfahrung : true))
            {
                wo.schätze.remove(hilf);
                schätze.add(hilf);
                if (hilf instanceof Spruch)
                {
                    ((Spruch) hilf).GefundenMelden();
                }
                InventarListeEintragen();
                RaumSchätzeEintragen();
            }
            else
            {
                spiel.BotschaftSenden("Soviel können Sie nicht mitnehmen, das Maximalgewicht ist: " + maxGewicht + (hilf instanceof Spruch ? (", das maximale Mentalgewicht: " + 20 + erfahrung) : ""));
            }
        }
    }
    
    /**
     * Trägt die Schätze des Raums in die Anzeige ein.
     */
    void RaumSchätzeEintragen()
    {
        int nummer;
        ArrayList<String> texte = new ArrayList<String>();
        nummer = 0;
        for (Schatz akt: wo.schätze)
        {
            texte.add(nummer + " " + akt.NameGeben() + " " + akt.gewicht);
            nummer = nummer + 1;
        }
        spiel.anzeige.RaumSchätzeSetzen(texte);
    }
    
    /**
     * Trägt die Daten des Spielers in die Anzeige ein.
     */
    void InventarListeEintragen()
    {
        int nummer;
        ArrayList<String> texte = new ArrayList<String>();
        double gewicht;
        gewicht = 0.0;
        for (Schatz akt: schätze)
        {
            gewicht += akt.GewichtGeben();
        }
        spiel.anzeige.SchlüsselAnzahlSetzen(AnzahlSchlüsselGeben());
        spiel.anzeige.GesamtgewichtSetzen(gewicht);
        nummer = 0;
        for (Schatz akt: schätze)
        {
            texte.add(nummer + " " + akt.NameGeben() + " " + akt.gewicht);
            nummer = nummer + 1;
        }
        spiel.anzeige.SpielerSchätzeSetzen(texte);
        if (handWaffe == null)
        {
            spiel.anzeige.HandwaffenNameSetzen("keine");
        }
        else
        {
            spiel.anzeige.HandwaffenNameSetzen(handWaffe.NameGeben());
        }
        if (schutz == null)
        {
            spiel.anzeige.SchutzwaffenNameSetzen("keine");
        }
        else
        {
            spiel.anzeige.SchutzwaffenNameSetzen(schutz.NameGeben());
        }
    }
    
    /**
     * Aktiviert den Schatz gemäß der Anweisung
     */
    void SchatzAktivieren()
    {
        Schatz hilf = schätze.get(spiel.kontroller.KommandoParameterGeben());
        if (hilf instanceof AngriffsWaffe)
        {
            handWaffe = (AngriffsWaffe) hilf;
            spiel.anzeige.HandwaffenNameSetzen(handWaffe.NameGeben());
        }
        else if (hilf instanceof SchutzWaffe)
        {
            schutz = (SchutzWaffe) hilf;
            spiel.anzeige.SchutzwaffenNameSetzen(schutz.NameGeben());
        }
        else if (hilf instanceof Spruch)
        {
            ((Spruch) hilf).Ausführen();
        }
    }
    
    /**
     * Ändert die Stärke um den angegebenen Betrag.
     * Die Grenzen für die Stärke werden sicher gestellt.
     * @param wert der Wert, um den sich die Stärke (höchstens) ändert
     */
    void StärkeÄndern(int wert)
    {
        stärke += wert;
        if (stärke < 0)
        {
            stärke = 0;
        }
        else if (stärke > maxStärke)
        {
            stärke = maxStärke; 
        }
        spiel.anzeige.StärkeSetzen(stärke);
    }

    /**
     * Entscheidet beim Monsterkampf über Weitermachen.
     * @return wahr, wenn weitergekämpft werden soll.
     */
    boolean KampfEntscheidTreffen()
    {
        SchätzeTesten();
        InventarListeEintragen();
        spiel.anzeige.RundeVerlorenSetzen(wo.monster.NameGeben());
        do
        {
            switch (spiel.kontroller.KommandoGeben())
            {
                case keinKommando:
                    break;
                case weiterKämpfen:
                    return true;
                case kampfAbbrechen:
                    return false;
                case handwaffeWeglegen:
                    handWaffe = null;
                    spiel.anzeige.HandwaffenNameSetzen("keine");
                    break;
                case schutzwaffeWeglegen:
                    schutz = null;
                    spiel.anzeige.SchutzwaffenNameSetzen("keine");
                    break;
                case schatzAktivieren:
                    SchatzAktivieren();
                    break;
                default:
                    System.out.println("Unerwartetes Kommando");
            }
        } while (true);
    }
    
    /**
     * Bestimmt den nächsten Raum
     * Vorher können Waffen und Spürüche gewählt werden.
     * @return wahr, wenn Spielende errecht wurde
     */
    boolean ZimmerEntscheidTreffen()
    {
        switch (spiel.kontroller.KommandoGeben())
        {
            case keinKommando:
                break;
            case spielAbbrechen:
                return true;
            case zimmerWechsel:
                boolean besiegt;
                int raum = spiel.kontroller.KommandoParameterGeben();
                spiel.BotschaftSenden("Sie gehen nach: " + raum);
                if (NachRaumGehen(raum))
                {
                    boolean monsterZählt = !(wo.monster instanceof LeeresMonster);
                    wo.monster.KampfBeginnen(this);
                    do
                    {
                        if (monsterZählt)
                        {
                            erfahrung = erfahrung + 1;
                        }
                        besiegt = wo.monster.KampfDurchführen(this);
                    } while (!besiegt && KampfEntscheidTreffen());
                    if (monsterZählt && besiegt)
                    {
                        erfahrung = erfahrung + 2;
                    }
                    spiel.anzeige.KampfZuEndeSetzen();
                    spiel.gewölbe.AnzeigeZeichnen(false, wo);
                    if (besiegt)
                    {
                        SchätzeTesten();
                        InventarListeEintragen();
                        if (spiel.gewölbe.AusgangTesten(wo) && (AnzahlSchlüsselGeben() == 3))
                        {
                            spiel.anzeige.EndemeldungAnzeigen();
                            spiel.kontroller.KommandoGeben();
                            return true;
                        }
                        else
                        {
                            if (!wo.SchatzVorhanden())
                            {
                                StärkeÄndern(1);
                            }
                            RaumSchätzeEintragen();
                        }
                    }
                    else
                    {
                        wo.monster.KampfAbbrechen(this);
                        SchätzeTesten();
                        InventarListeEintragen();
                    }
                }
                else
                {
                    spiel.BotschaftSenden("Diesen Raum können Sie nicht erreichen");
                }
                break;
            case handwaffeWeglegen:
                handWaffe = null;
                spiel.anzeige.HandwaffenNameSetzen("keine");
                break;
            case schutzwaffeWeglegen:
                schutz = null;
                spiel.anzeige.SchutzwaffenNameSetzen("keine");
                break;
            case schatzAktivieren:
                SchatzAktivieren();
                break;
            case schatzDalassen:
                SchatzDalassen(spiel.kontroller.KommandoParameterGeben());
                break;
            case schatzMitnehmen:
                SchatzMitnehmen(spiel.kontroller.KommandoParameterGeben());
                break;
            default:
                System.out.println("Unerwartetes Kommando");
        }
        return false;
    }
}
