
import java.util.ArrayList;
/**
 * Verwaltung des Hauses
 *
 * @author Albert Wiedemann
 * @version 1.0
 */
class Haus
{
    private HintergrundSymbol hintergrund;
    private Thermometer thermometerEsszimmer;
    private HeizkoerperSymbol heizkörperSymbolEsszimmer;
    private Heizkoerper heizkörperEsszimmer;
    private Uhr uhrHaus;
    private Windsensor windsensorHaus;
    private RollladenSymbol rollladenSymbolWohnzimmer1;
    private Rollladen rollladenWohnzimmer1;
    private RollladenSymbol rollladenSymbolWohnzimmer2;
    private Rollladen rollladenWohnzimmer2;
    private RollladenSymbol rollladenSymbolEsszimmer;
    private Rollladen rollladenEsszimmer;
    private RollladenSymbol rollladenSymbolKüche;
    private Rollladen rollladenKüche;
    private Bewegungsmelder bewegungsmelderWohnzimmer2;
    private Sonnensensor sonnensensorWohnzimmer2;
    private LampenSymbol lampenSymbolWohnzimmer2_1;
    private Lampe lampeWohnzimmer2_1;
    private LampenSymbol lampenSymbolWohnzimmer2_2;
    private Lampe lampeWohnzimmer2_2;
    private Waage waageKüche;
    private KuehlschrankSymbol kühlschrankSymbolKüche;
    private Kuehlschrank kühlschrankKüche;
    private ArrayList<Rollladen> rollläden;
    private ArrayList<Lampe> lampen;

    /**
     * Legt die Sensoren und Komponenten des Hauses an
     */
    Haus()
    {
        rollläden = new ArrayList<Rollladen>();
        lampen = new ArrayList<Lampe>();

        hintergrund = new HintergrundSymbol();
        thermometerEsszimmer = new Thermometer("Esszimmer");
        heizkörperSymbolEsszimmer = new HeizkoerperSymbol(420, 440);
        heizkörperEsszimmer = new Heizkoerper("Esszimmer", heizkörperSymbolEsszimmer, thermometerEsszimmer);
        uhrHaus = new Uhr("Haus");
        windsensorHaus = new Windsensor("Haus");
        rollladenSymbolWohnzimmer1 = new RollladenSymbol(250, 125);
        rollladenWohnzimmer1 = new Rollladen("Wohnzimmer1", rollladenSymbolWohnzimmer1, windsensorHaus, uhrHaus);
        rollläden.add(rollladenWohnzimmer1);
        rollladenSymbolWohnzimmer2 = new RollladenSymbol(500, 125);
        rollladenWohnzimmer2 = new Rollladen("Wohnzimmer2", rollladenSymbolWohnzimmer2, windsensorHaus, uhrHaus);
        rollläden.add(rollladenWohnzimmer2);
        rollladenSymbolEsszimmer = new RollladenSymbol(500, 325);
        rollladenEsszimmer = new Rollladen("Esszimmer", rollladenSymbolEsszimmer, windsensorHaus, uhrHaus);
        rollläden.add(rollladenEsszimmer);
        rollladenSymbolKüche = new RollladenSymbol(250, 325);
        rollladenKüche = new Rollladen("Küche", rollladenSymbolKüche, windsensorHaus, uhrHaus);
        rollläden.add(rollladenKüche);
        sonnensensorWohnzimmer2 = new Sonnensensor("Wohnzimmer2");
        bewegungsmelderWohnzimmer2 = new Bewegungsmelder("Wohnzimmer2");
        lampenSymbolWohnzimmer2_1 = new LampenSymbol(450, 105);
        lampeWohnzimmer2_1 = new Lampe("Wohnzimmer2 1", lampenSymbolWohnzimmer2_1, bewegungsmelderWohnzimmer2, sonnensensorWohnzimmer2);
        lampen.add(lampeWohnzimmer2_1);
        lampenSymbolWohnzimmer2_2 = new LampenSymbol(550, 105);
        lampeWohnzimmer2_2 = new Lampe("Wohnzimmer2 2", lampenSymbolWohnzimmer2_2, bewegungsmelderWohnzimmer2, sonnensensorWohnzimmer2);
        lampen.add(lampeWohnzimmer2_2);
        waageKüche = new Waage("Küche");
        kühlschrankSymbolKüche = new KuehlschrankSymbol(158, 370);
        kühlschrankKüche = new Kuehlschrank("Küche", kühlschrankSymbolKüche, waageKüche);
    }
    
    /**
     * Setzt das Thermometer im Esszimmer auf einen neuen Wert
     * @param eingabe neuer Temperaturwert
     */
    void ThermometerEsszimmerWertSetzen (int eingabe)
    {
        thermometerEsszimmer.WertSetzen(eingabe);
    }
    
    /**
     * Setzt Uhr im Haus auf eine neue Zeit
     * @param stunden neue Stundenangabe
     */
    void UhrHausWertSetzen (int stunden)
    {
        uhrHaus.WertSetzen(stunden);
    }
    
    /**
     * Setzt Windsensor im Haus auf einen neuen Wert
     * @param eingabe neue Windstärke
     */
    void WindsensorHausWertSetzen (int eingabe)
    {
        windsensorHaus.WertSetzen(eingabe);
    }
    
    /**
     * Setzt Bewegunsmelder in Wohnzimmer 2 auf einen neuen Wert
     * @param an neuer Wert
     */
    void BewegungsmelderWohnzimmer2WertSetzen (boolean an)
    {
        bewegungsmelderWohnzimmer2.WertSetzen(an);
    }
    
    /**
     * Setzt Sonnensensor in Wohnzimmer 2 auf einen neuen Wert
     * @param eingabe neue Helligkeit
     */
    void SonnensensorWohnzimmer2WertSetzen (int eingabe)
    {
        sonnensensorWohnzimmer2.WertSetzen(eingabe);
    }
    
    /**
     * Setzt Waage in der Küche auf einen neuen Wert
     * @param eingabe neues Gewicht
     */
    void WaageKücheWertSetzen (int eingabe)
    {
        waageKüche.WertSetzen(eingabe);
    }
    
    /**
     * Schaltet den Heizkörper im Esszimmer dauerhaft ein
     */
    void HeizkörperEsszimmerEinschalten()
    {
        heizkörperEsszimmer.Einschalten();
    }
    
    /**
     * Schaltet den Heizkörper im Esszimmer dauerhaft aus
     */
    void HeizkörperEsszimmerAusschalten()
    {
        heizkörperEsszimmer.Ausschalten();
    }
    
    /**
     * Setzt den Heizkörper im Esszimmer auf Automatikbetrieb
     */
    void HeizkörperEsszimmerAutomatikSetzen()
    {
        heizkörperEsszimmer.AutomatikAktivieren();
    }
    
    /**
     * Schaltet die Lampe 1 in Wohnzimmer 2 dauerhaft ein
     */
    void LampeWohnzimmer2_1Einschalten()
    {
        lampeWohnzimmer2_1.Einschalten();
    }
    
    /**
     * Schaltet die Lampe 1 in Wohnzimmer 2 dauerhaft aus
     */
    void LampeWohnzimmer2_1Ausschalten()
    {
        lampeWohnzimmer2_1.Ausschalten();
    }
    
    /**
     * Setzt die Lampe 1 in Wohnzimmer 2 auf Automatikbetrieb
     */
    void LampeWohnzimmer2_1AutomatikSetzen()
    {
        lampeWohnzimmer2_1.AutomatikAktivieren();
    }
    
    /**
     * Schaltet die Lampe 2 in Wohnzimmer 2 dauerhaft ein
     */
    void LampeWohnzimmer2_2Einschalten()
    {
        lampeWohnzimmer2_2.Einschalten();
    }
    
    /**
     * Schaltet die Lampe 2 in Wohnzimmer 2 dauerhaft aus
     */
    void LampeWohnzimmer2_2Ausschalten()
    {
        lampeWohnzimmer2_2.Ausschalten();
    }
    
    /**
     * Setzt die Lampe 2 in Wohnzimmer 2 auf Automatikbetrieb
     */
    void LampeWohnzimmer2_2AutomatikSetzen()
    {
        lampeWohnzimmer2_2.AutomatikAktivieren();
    }
    
    /**
     * Zieht Rollladen in Wohnzimmer 1 dauerhaft hoch
     */
    void RollladenWohnzimmer1Hochziehen()
    {
        rollladenWohnzimmer1.Hochziehen();
    }
    
    /**
     * Lässt Rollladen in Wohnzimmer 1 dauerhaft runter
     */
    void RollladenWohnzimmer1Runterlassen()
    {
        rollladenWohnzimmer1.Runterlassen();
    }
    
    /**
     * Setzt Rollladen in Wohnzimmer 1 auf Automatikbetrieb
     */
    void RollladenWohnzimmer1AutomatikSetzen()
    {
        rollladenWohnzimmer1.AutomatikAktivieren();
    }
    
    /**
     * Zieht Rollladen in Wohnzimmer 2 dauerhaft hoch
     */
    void RollladenWohnzimmer2Hochziehen()
    {
        rollladenWohnzimmer2.Hochziehen();
    }
    
    /**
     * Lässt Rollladen in Wohnzimmer 2 dauerhaft runter
     */
    void RollladenWohnzimmer2Runterlassen()
    {
        rollladenWohnzimmer2.Runterlassen();
    }
    
    /**
     * Setzt Rollladen in Wohnzimmer 2 auf Automatikbetrieb
     */
    void RollladenWohnzimmer2AutomatikSetzen()
    {
        rollladenWohnzimmer2.AutomatikAktivieren();
    }
    
    /**
     * Zieht Rollladen im Esszimmer dauerhaft hoch
     */
    void RollladenEsszimmerHochziehen()
    {
        rollladenEsszimmer.Hochziehen();
    }
    
    /**
     * Lässt Rollladen im Esszimmer dauerhaft runter
     */
    void RollladenEsszimmerRunterlassen()
    {
        rollladenEsszimmer.Runterlassen();
    }
    
    /**
     * Setzt Rollladen im Esszimmer auf Automatikbetrieb
     */
    void RollladenEsszimmerAutomatikSetzen()
    {
        rollladenEsszimmer.AutomatikAktivieren();
    }
    
    /**
     * Zieht Rollladen in der Küche dauerhaft hoch
     */
    void RollladenKücheHochziehen()
    {
        rollladenKüche.Hochziehen();
    }
    
    /**
     * Lässt Rollladen in der Küche dauerhaft runter
     */
    void RollladenKücheRunterlassen()
    {
        rollladenKüche.Runterlassen();
    }
    
    /**
     * Setzt Rollladen in der Küche auf Automatikbetrieb
     */
    void RollladenKücheAutomatikSetzen()
    {
        rollladenKüche.AutomatikAktivieren();
    }
    
    /**
     * Schaltet alle Lampen dauerhaft ein
     */
    void LampenEinschalten()
    {
        for (Lampe l: lampen)
        {
            l.Einschalten();
        }
    }
    
    /**
     * Schaltet alle Lampen dauerhaft aus
     */
    void LampenAusschalten()
    {
        for (Lampe l: lampen)
        {
            l.Ausschalten();
        }
    }
    
    /**
     * Setzt alle Lampen auf Automatikbetrieb
     */
    void LampenAutomatikSetzen()
    {
        for (Lampe l: lampen)
        {
            l.AutomatikAktivieren();
        }
    }
    
    /**
     * Zieht alle Rollläden dauerhaft hoch
     */
    void RollladenHochziehen()
    {
        for (Rollladen r: rollläden)
        {
            r.Hochziehen();
        }
    }
    
    /**
     * Lässt alle Rollläden dauerhaft runter
     */
    void RollladenRunterlassen()
    {
        for (Rollladen r: rollläden)
        {
            r.Runterlassen();
        }
    }
    
    /**
     * Setzt alle Rollläden auf Automatikbetrieb
     */
    void RollladenAutomatikSetzen()
    {
        for (Rollladen r: rollläden)
        {
            r.AutomatikAktivieren();
        }
    }

    /**
     * Zeigt die zustände aller Geräte an
     */
    void ZuständeAnzeigen()
    {
        System.out.println(thermometerEsszimmer.NameGeben() + ": " + thermometerEsszimmer.ZustandGeben());
        System.out.println(heizkörperEsszimmer.NameGeben() + ": " + heizkörperEsszimmer.ZustandGeben());
        System.out.println(uhrHaus.NameGeben() + ": " + uhrHaus.ZustandGeben());
        System.out.println(windsensorHaus.NameGeben() + ": " + windsensorHaus.ZustandGeben());
        System.out.println(rollladenWohnzimmer1.NameGeben() + ": " + rollladenWohnzimmer1.ZustandGeben());
        System.out.println(rollladenWohnzimmer2.NameGeben() + ": " + rollladenWohnzimmer2.ZustandGeben());
        System.out.println(rollladenEsszimmer.NameGeben() + ": " + rollladenEsszimmer.ZustandGeben());
        System.out.println(rollladenKüche.NameGeben() + ": " + rollladenKüche.ZustandGeben());
        System.out.println(sonnensensorWohnzimmer2.NameGeben() + ": " + sonnensensorWohnzimmer2.ZustandGeben());
        System.out.println(bewegungsmelderWohnzimmer2.NameGeben() + ": " + bewegungsmelderWohnzimmer2.ZustandGeben());
        System.out.println(lampeWohnzimmer2_1.NameGeben() + ": " + lampeWohnzimmer2_1.ZustandGeben());
        System.out.println(lampeWohnzimmer2_2.NameGeben() + ": " + lampeWohnzimmer2_2.ZustandGeben());
        System.out.println(waageKüche.NameGeben() + ": " + waageKüche.ZustandGeben());
        System.out.println(kühlschrankKüche.NameGeben() + ": " + kühlschrankKüche.ZustandGeben());
    }
}
