
/**
 * Beschreibt einen Heizkörper
 * 
 * @author Albert Wiedemann
 * @version 1.0
 */
class Heizkoerper extends Hausgeraet
{
    /** Zustand des Heizkörpers */
    private boolean an;
    /** Wert des Thermometers */
    private int temperatur;
    /** Darstellungssymbol */
    private HeizkoerperSymbol symbol;

    /**
     * Legt den Namen und das Darstellungssymbol fest
     * @param namensZusatz individualler Namensteil der Komponente
     * @param sy Darstellungssymbol
     * @param t Thermometer
     */
    Heizkoerper(String namensZusatz, HeizkoerperSymbol sy, Thermometer t)
    {
        super("Heizkörper " + namensZusatz);
        temperatur = 0;
        an = true;
        automatik = true;
        symbol = sy;
        symbol.ÄnderungMelden(an);
        t.BeobachterRegistrieren(this);
    }

    /**
     * Wird vom Thermometer bei einer neuen Temperatur aufgerufen
     * @param wert die neue Temperatur
     * @param wer das aufrufende Messgerät
     */
    @Override void WertänderungMelden(int wert, Messgeraet wer)
    {
        temperatur = wert;
        if (automatik)
        {
            an = temperatur < 22;
            symbol.ÄnderungMelden(an);
        }
    }
    
    /**
     * Meldet den Zustand der Komponente
     * @return Zustand als Zeichenkette
     */
    @Override String ZustandGeben()
    {
        return an ? "ein" : "aus";
    }
    
    /**
     * Schaltet die Komponente dauerhaft ein
     */
    @Override void Einschalten()
    {
        an = true;
        automatik = false;
        symbol.ÄnderungMelden(an);
    }
    
    /**
     * Schaltet die Komponente dauerhaft aus
     */
    @Override void Ausschalten()
    {
        an = false;
        automatik = false;
        symbol.ÄnderungMelden(an);
    }
    
    /**
     * Schaltet die Komponente auf Automatikbetrieb
     */
    @Override void AutomatikAktivieren()
    {
        an = temperatur < 22;
        automatik = true;
        symbol.ÄnderungMelden(an);
    }
}
