
/**
 * Beschreibt eine Lampe
 * 
 * @author Albert Wiedemann
 * @version 1.0
 */
class Lampe extends Hausgeraet
{
    /** Zustand der Lampe */
    private boolean an;
    /** Wert des Bewegungsmelders */
    private boolean bewegung;
    /** Wert des Sonnensensors */
    private int helligkeit;
    /** der Bewegungsmelder */
    private Bewegungsmelder b;
    /** der Sonnensensor */
    private Sonnensensor s;
    /** Darstellungssymbol */
    private LampenSymbol symbol;

    /**
     * Legt den Namen und das Darstellungssymbol fest
     * @param namensZusatz individualler Namensteil der Komponente
     * @param sy Darstellungssymbol
     * @param b Bewegungsmelder
     * @param s Sonnensensor
     */
    Lampe(String namensZusatz, LampenSymbol sy, Bewegungsmelder b, Sonnensensor s)
    {
        super("Lampe " + namensZusatz);
        bewegung = false;
        helligkeit = 0;
        an = false;
        automatik = true;
        symbol = sy;
        symbol.ÄnderungMelden(an);
        s.BeobachterRegistrieren(this);
        b.BeobachterRegistrieren(this);
        this.b = b;
        this.s = s;
    }

    /**
     * Wird von Sonnensensor bei einem neuen Helligkeitswert aufgerufen
     * @param wert der neue Helligkeitswert
     * @param wer das aufrufende Messgerät
     */
    @Override void WertänderungMelden(int wert, Messgeraet wer)
    {
        if (wer == s)
        {
            helligkeit = wert;
            if (automatik)
            {
                an = bewegung && (helligkeit < 30);
                symbol.ÄnderungMelden(an);
            }
        }
    }

    /**
     * Wird von Bewegungsmelder bei einem neuen Bewegungszustand aufgerufen
     * @param wert der neue Bewegungszustand
     * @param wer das aufrufende Messgerät
     */
    @Override void WertänderungMelden(boolean wert, Messgeraet wer)
    {
        if (wer == b)
        {
            bewegung = wert;
            if (automatik)
            {
                an = bewegung && (helligkeit < 30);
                symbol.ÄnderungMelden(an);
            }
        }
    }
    
    /**
     * Meldet den Zustand der Komponente
     * @return Zustand als Zeichenkette
     */
    @Override String ZustandGeben()
    {
        return an ? "ein" : "aus";
    }
    
    /**
     * Schaltet die Komponente dauerhaft ein
     */
    @Override void Einschalten()
    {
        super.Einschalten();
        an = true;
        symbol.ÄnderungMelden(an);
    }
    
    /**
     * Schaltet die Komponente dauerhaft aus
     */
    @Override void Ausschalten()
    {
        super.Ausschalten();
        an = false;
        symbol.ÄnderungMelden(an);
    }
    
    /**
     * Schaltet die Komponente auf Automatikbetrieb
     */
    @Override void AutomatikAktivieren()
    {
        super.AutomatikAktivieren();
        an = bewegung && (helligkeit < 30);
        symbol.ÄnderungMelden(an);
    }
}
