
/**
 * Beschreibt einen Rollladen
 * 
 * @author Albert Wiedemann
 * @version 1.0
 */
class Rollladen
{
    /** Name der Hauskomponente */
    private String name;
    /** Zustand des Rollladens */
    private boolean oben;
    /** Automatikbetrieb */
    private boolean automatik;
    /** Wert des Windsensors */
    private int windwert;
    /** Wert der Uhr */
    private int uhrzeit;
    /** Darstellungssymbol */
    private RollladenSymbol symbol;

    /**
     * Legt den Namen und das Darstellungssymbol fest
     * @param namensZusatz individualler Namensteil der Komponente
     * @param sy Darstellungssymbol
     * @param w Windsensor
     * @param u Uhr
     */
    Rollladen(String namensZusatz, RollladenSymbol sy, Windsensor w, Uhr u)
    {
        name = "Rollladen " + namensZusatz;
        oben = false;
        automatik = true;
        symbol = sy;
        symbol.ÄnderungMelden(oben);
        w.BeobachterRegistrieren(this);
        u.BeobachterRegistrieren(this);
    }
    
    /**
     * Meldet den Namen des Geräts zurück
     * @return Name
     */
    String NameGeben()
    {
        return name;
    }

    /**
     * Wird vom Windsensor bei einer neuen Windstärke aufgerufen
     * @param wert die neue Windstärke
     */
    void WertänderungMeldenWindSensor(int wert)
    {
        windwert = wert;
        if (automatik)
        {
            oben = (windwert > 6) || (uhrzeit > 60000) && (uhrzeit < 200000);
            symbol.ÄnderungMelden(oben);
        }
    }

    /**
     * Wird von der Uhr bei einer neuen Uhrzeit aufgerufen
     * @param wert die neue Uhrzeit
     */
    void WertänderungMeldenUhr(int wert)
    {
        uhrzeit = wert;
        if (automatik)
        {
            oben = (windwert > 6) || (uhrzeit > 60000) && (uhrzeit < 200000);
            symbol.ÄnderungMelden(oben);
        }
    }
        
    /**
     * Meldet den Zustand der Komponente
     * @return Zustand als Zeichenkette
     */
    String ZustandGeben()
    {
        return oben ? "offen" : "geschlossen";
    }

    /**
     * Zieht den Rollladen dauerhaft hoch
     */
    void Hochziehen()
    {
        oben = true;
        automatik = false;
        symbol.ÄnderungMelden(oben);
    }
    
    /**
     * lässt den Rollladen dauerhaft runter
     */
    void Runterlassen()
    {
        oben = false;
        automatik = false;
        symbol.ÄnderungMelden(oben);
    }
    
    /**
     * Schaltet die Komponente auf Automatikbetrieb
     */
    void AutomatikAktivieren()
    {
        oben = (windwert > 6) || (uhrzeit > 60000) && (uhrzeit < 200000);
        automatik = true;
        symbol.ÄnderungMelden(oben);
    }
}
